import * as tracing_1 from "@effect-ts/core/Tracing";
const fileName_1 = "packages/@contentlayer/source-files-stackbit/src/index.ts";
import * as SourceFiles from '@contentlayer/source-files';
import { casesHandled, isReadonlyArray, not, notImplemented, partition, pick } from '@contentlayer/utils';
import { identity } from '@contentlayer/utils/effect';
export { addComputedFields } from './addComputedFields.js';
export const stackbitConfigToDocumentTypes = (stackbitConfig) => {
    const models = stackbitConfig.models.filter(not(isImageModel));
    const [documentLikeModels, objectModels] = partition(models, isDocumentLikeModel);
    const ctx = { documentTypeMap: {}, nestedTypeMap: {} };
    const documentTypes = documentLikeModels.map(stackbitDocumentLikeModelToDocumentType(ctx));
    objectModels.forEach((model) => {
        const nestedType = stackbitObjectModelToDocumentType(ctx)(model);
        ctx.nestedTypeMap[model.name] = nestedType;
    });
    documentTypes.forEach((documentType) => {
        ctx.documentTypeMap[documentType.def().name] = documentType;
    });
    return documentTypes;
};
const stackbitDocumentLikeModelToDocumentType = (ctx) => (stackbitModel) => {
    return SourceFiles.defineDocumentType(() => ({
        name: stackbitModel.name,
        description: stackbitModel.description,
        fields: (stackbitModel.fields ?? []).map(stackbitFieldToField(ctx)),
        isSingleton: stackbitModel.type === 'config' || stackbitModel.singleInstance === true,
    }));
};
const stackbitObjectModelToDocumentType = (ctx) => (stackbitModel) => {
    return SourceFiles.defineNestedType(() => ({
        name: stackbitModel.name,
        description: stackbitModel.description,
        fields: (stackbitModel.fields ?? []).map(stackbitFieldToField(ctx)),
    }));
};
const stackbitFieldToField = (ctx) => (stackbitField) => {
    const commonFields = {
        ...pick(stackbitField, ['name', 'description', 'required']),
        default: stackbitField.default,
    };
    switch (stackbitField.type) {
        case 'boolean':
        case 'number':
        case 'string':
            return { ...commonFields, type: stackbitField.type };
        case 'enum':
            return {
                ...commonFields,
                type: 'enum',
                options: stackbitField.options.map(mapStackbitEnumOption),
            };
        case 'style':
            return { ...commonFields, type: 'json' };
        case 'list': {
            const of = stackbitListItemToListFieldDef(ctx)(stackbitField.items);
            return isReadonlyArray(of)
                ? {
                    ...commonFields,
                    type: 'list',
                    of,
                    typeField: 'type',
                }
                : { ...commonFields, type: 'list', of };
        }
        case 'reference': {
            const of = stackbitField.models.map((modelName) => ctx.documentTypeMap[modelName]);
            if (of.length === 1) {
                return { ...commonFields, type: 'reference', of: of[0] };
            }
            return {
                ...commonFields,
                type: 'reference',
                of,
                typeField: 'type',
            };
        }
        case 'model': {
            const of = stackbitField.models.map((modelName) => ctx.nestedTypeMap[modelName]);
            if (of.length === 1) {
                return { ...commonFields, type: 'nested', of: of[0] };
            }
            return {
                ...commonFields,
                type: 'nested',
                of,
                typeField: 'type',
            };
        }
        case 'object': {
            const unnamedNestedTypeDef = {
                fields: stackbitField.fields.map(stackbitFieldToField(ctx)),
            };
            return {
                ...commonFields,
                type: 'nested',
                of: { type: 'nested', def: () => unnamedNestedTypeDef },
            };
        }
        case 'json':
            return { ...commonFields, type: 'json' };
        case 'url':
        case 'text':
        case 'image':
        case 'color':
        case 'markdown':
        case 'slug':
        case 'html':
        case 'date':
        case 'file':
        case 'richText':
        case 'datetime':
            return { ...commonFields, type: 'string' };
        default:
            casesHandled(stackbitField);
    }
};
const stackbitListItemToListFieldDef = (ctx) => (stackbitListItem) => {
    switch (stackbitListItem.type) {
        case 'boolean':
        case 'string':
            return { type: stackbitListItem.type };
        case 'enum':
            return {
                type: 'enum',
                options: stackbitListItem.options.map(mapStackbitEnumOption),
            };
        case 'reference':
            return firstArrayItemIfOne(stackbitListItem.models.map((modelName) => ctx.documentTypeMap[modelName]));
        case 'model':
            return firstArrayItemIfOne(stackbitListItem.models.map((modelName) => ctx.nestedTypeMap[modelName]));
        default:
            notImplemented(stackbitListItem.type);
    }
};
const mapStackbitEnumOption = (option) => {
    if (typeof option === 'string' || typeof option === 'number') {
        return option.toString();
    }
    return option.value.toString();
};
const isDocumentLikeModel = (model) => model.type === 'data' || model.type === 'page' || model.type === 'config';
const isImageModel = (model) => model.type === 'image';
const firstArrayItemIfOne = (array) => (array.length === 1 ? array[0] : array);
//# sourceMappingURL=index.js.map